@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2017 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@ class CControllerMediatypeCreate extends CController {
 	protected function checkInput() {
 		$fields = [
 			'type' =>					'required|db media_type.type|in '.implode(',', array_keys(media_type2str())),
-			'description' =>			'db media_type.description|not_empty',
+			'name' =>					'db media_type.name|not_empty',
 			'smtp_server' =>			'db media_type.smtp_server',
 			'smtp_port' =>				'db media_type.smtp_port',
 			'smtp_helo' =>				'db media_type.smtp_helo',
@@ -34,18 +34,24 @@ class CControllerMediatypeCreate extends CController {
 			'smtp_verify_host' =>		'db media_type.smtp_verify_host|in 0,1',
 			'smtp_authentication' =>	'db media_type.smtp_authentication|in '.SMTP_AUTHENTICATION_NONE.','.SMTP_AUTHENTICATION_NORMAL,
 			'exec_path' =>				'db media_type.exec_path',
-			'eztext_limit' =>			'in '.EZ_TEXTING_LIMIT_USA.','.EZ_TEXTING_LIMIT_CANADA,
 			'exec_params' =>			'array',
 			'gsm_modem' =>				'db media_type.gsm_modem',
-			'jabber_username' =>		'db media_type.username',
-			'eztext_username' =>		'db media_type.username',
 			'smtp_username' =>			'db media_type.username',
 			'passwd' =>					'db media_type.passwd',
+			'parameters' =>				'array',
+			'script' => 				'db media_type.script',
+			'timeout' => 				'db media_type.timeout',
+			'process_tags' =>			'in '.ZBX_MEDIA_TYPE_TAGS_DISABLED.','.ZBX_MEDIA_TYPE_TAGS_ENABLED,
+			'show_event_menu' =>		'in '.ZBX_EVENT_MENU_HIDE.','.ZBX_EVENT_MENU_SHOW,
+			'event_menu_url' =>			'db media_type.event_menu_url',
+			'event_menu_name' =>		'db media_type.event_menu_name',
 			'status' =>					'db media_type.status|in '.MEDIA_TYPE_STATUS_ACTIVE.','.MEDIA_TYPE_STATUS_DISABLED,
 			'maxsessions' =>			'db media_type.maxsessions',
 			'maxattempts' =>			'db media_type.maxattempts',
 			'attempt_interval' =>		'db media_type.attempt_interval',
-			'form_refresh' =>			'int32'
+			'description' =>			'db media_type.description',
+			'form_refresh' =>			'int32',
+			'content_type' =>			'db media_type.content_type|in '.SMTP_MESSAGE_FORMAT_PLAIN_TEXT.','.SMTP_MESSAGE_FORMAT_HTML
 		];
 
 		$ret = $this->validateInput($fields);
@@ -86,12 +92,14 @@ class CControllerMediatypeCreate extends CController {
 	protected function doAction() {
 		$mediatype = [];
 
-		$this->getInputs($mediatype, ['type', 'description', 'status', 'maxsessions', 'maxattempts', 'attempt_interval']);
+		$this->getInputs($mediatype, ['type', 'name', 'status', 'maxsessions', 'maxattempts', 'attempt_interval',
+			'description'
+		]);
 
 		switch ($mediatype['type']) {
 			case MEDIA_TYPE_EMAIL:
 				$this->getInputs($mediatype, ['smtp_server', 'smtp_port', 'smtp_helo', 'smtp_email', 'smtp_security',
-					'smtp_verify_peer', 'smtp_verify_host', 'smtp_authentication', 'passwd'
+					'smtp_verify_peer', 'smtp_verify_host', 'smtp_authentication', 'passwd', 'content_type'
 				]);
 
 				if ($this->hasInput('smtp_username')) {
@@ -118,37 +126,27 @@ class CControllerMediatypeCreate extends CController {
 				$mediatype['maxsessions'] = 1;
 				break;
 
-			case MEDIA_TYPE_JABBER:
-				$this->getInputs($mediatype, ['passwd']);
+			case MEDIA_TYPE_WEBHOOK:
+				$mediatype['process_tags'] = ZBX_MEDIA_TYPE_TAGS_DISABLED;
+				$mediatype['show_event_menu'] = ZBX_EVENT_MENU_HIDE;
+				$this->getInputs($mediatype, ['script', 'timeout', 'process_tags', 'show_event_menu', 'event_menu_url',
+					'event_menu_name'
+				]);
+				$parameters = $this->getInput('parameters', []);
 
-				if ($this->hasInput('jabber_username')) {
-					$mediatype['username'] = $this->getInput('jabber_username');
-				}
-				break;
-
-			case MEDIA_TYPE_EZ_TEXTING:
-				$this->getInputs($mediatype, ['passwd']);
-
-				if ($this->hasInput('eztext_username')) {
-					$mediatype['username'] = $this->getInput('eztext_username');
-				}
-
-				if ($this->hasInput('eztext_limit')) {
-					$mediatype['exec_path'] = $this->getInput('eztext_limit');
+				if (array_key_exists('name', $parameters) && array_key_exists('value', $parameters)) {
+					$mediatype['parameters'] = array_map(function ($name, $value) {
+							return compact('name', 'value');
+						},
+						$parameters['name'],
+						$parameters['value']
+					);
 				}
 				break;
 		}
 
-		DBstart();
-
 		$result = API::Mediatype()->create($mediatype);
 
-		if ($result) {
-			add_audit(AUDIT_ACTION_ADD, AUDIT_RESOURCE_MEDIA_TYPE, 'Media type ['.$mediatype['description'].']');
-		}
-
-		$result = DBend($result);
-
 		if ($result) {
 			$response = new CControllerResponseRedirect('zabbix.php?action=mediatype.list&uncheck=1');
 			$response->setMessageOk(_('Media type added'));
