@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2017 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,17 +19,15 @@
 **/
 
 
-require_once dirname(__FILE__).'/../../include/hostgroups.inc.php';
-
 class CControllerWidgetDataOverView extends CControllerWidget {
 
 	public function __construct() {
 		parent::__construct();
 
-		$this->setType(WIDGET_DATA_OVERVIEW);
+		$this->setType(WIDGET_DATA_OVER);
 		$this->setValidationRules([
 			'name' => 'string',
-			'fields' => 'json'
+			'fields' => 'json',
 		]);
 	}
 
@@ -40,6 +38,7 @@ class CControllerWidgetDataOverView extends CControllerWidget {
 			'name' => $this->getInput('name', $this->getDefaultHeader()),
 			'groupids' => getSubGroups($fields['groupids']),
 			'application' => $fields['application'],
+			'show_suppressed' => $fields['show_suppressed'],
 			'style' => $fields['style'],
 			'user' => [
 				'debug_mode' => $this->getDebugMode()
