@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2017 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -61,12 +61,13 @@ $data = [
 		'screens' => ['updateExisting' => false, 'createMissing' => false],
 		'maps' => ['updateExisting' => false, 'createMissing' => false],
 		'images' => ['updateExisting' => false, 'createMissing' => false],
+		'mediaTypes' => ['updateExisting' => false, 'createMissing' => false],
 		'valueMaps' => ['updateExisting' => false, 'createMissing' => false]
 	],
 	'backurl' => getRequest('backurl', '')
 ];
 
-if (!CHtmlUrlValidator::validate($data['backurl'], false, true)) {
+if (!CHtmlUrlValidator::validate($data['backurl'], ['allow_user_macro' => false, 'validate_uri_schemes' => true])) {
 	$data['backurl'] = 'zabbix.php?action=dashboard.view';
 }
 
@@ -116,6 +117,12 @@ if (hasRequest('rules_preset') && !hasRequest('rules')) {
 			$data['backurl'] = 'sysmaps.php';
 			break;
 
+		case 'mediatype':
+			$data['rules']['mediaTypes'] = ['updateExisting' => false, 'createMissing' => true];
+
+			$data['backurl'] = 'zabbix.php?action=mediatype.list';
+			break;
+
 		case 'screen':
 			$data['rules']['screens'] = ['updateExisting' => true, 'createMissing' => true];
 
