@@ -51,16 +51,16 @@ $fields = [
 	],
 	'pairs'           => [T_ZBX_STR, O_OPT, P_NO_TRIM,  null,                    null],
 	'steps'           => [T_ZBX_STR, O_OPT, P_NO_TRIM,  null,                    'isset({add}) || isset({update})', _('Steps')],
-	'authentication'  => [T_ZBX_INT, O_OPT, null,  IN('0,1,2'),             'isset({add}) || isset({update})'],
+	'authentication'  => [T_ZBX_INT, O_OPT, null,  IN('0,1,2,3'),             'isset({add}) || isset({update})'],
 	'http_user'       => [T_ZBX_STR, O_OPT, null,  null,
 		'(isset({add}) || isset({update})) && isset({authentication}) && ({authentication}=='.HTTPTEST_AUTH_BASIC.
-			' || {authentication}=='.HTTPTEST_AUTH_NTLM.
+			' || {authentication}=='.HTTPTEST_AUTH_NTLM.' || {authentication}=='.HTTPTEST_AUTH_KERBEROS.
 		')',
 		_('User')
 	],
 	'http_password'		=> [T_ZBX_STR, O_OPT, P_NO_TRIM, null,
 		'(isset({add}) || isset({update})) && isset({authentication}) && ({authentication}=='.HTTPTEST_AUTH_BASIC.
-			' || {authentication}=='.HTTPTEST_AUTH_NTLM.
+			' || {authentication}=='.HTTPTEST_AUTH_NTLM.' || {authentication}=='.HTTPTEST_AUTH_KERBEROS.
 		')',
 		_('Password')
 	],
@@ -698,14 +698,16 @@ if (isset($_REQUEST['form'])) {
 
 	$data['application_list'] = [];
 	if (!empty($data['hostid'])) {
-		$dbApps = DBselect(
-			'SELECT a.applicationid,a.name'.
-			' FROM applications a'.
-			' WHERE a.hostid='.zbx_dbstr($data['hostid']).
-				' AND a.flags='.ZBX_FLAG_DISCOVERY_NORMAL
-		);
-		while ($dbApp = DBfetch($dbApps)) {
-			$data['application_list'][$dbApp['applicationid']] = $dbApp['name'];
+		$db_applications = API::Application()->get([
+			'output' => ['name'],
+			'hostids' => $data['hostid'],
+			'filter' => ['flags' => ZBX_FLAG_DISCOVERY_NORMAL],
+			'preservekeys' => true
+		]);
+
+		CArrayHelper::sort($db_applications, ['name']);
+		foreach ($db_applications as $applicationid => $db_application) {
+			$data['application_list'][$applicationid] = $db_application['name'];
 		}
 	}
 
