@@ -41,7 +41,6 @@ class CScreenItem extends CApiService {
 		SCREEN_RESOURCE_TRIGGER_INFO,
 		SCREEN_RESOURCE_SERVER_INFO,
 		SCREEN_RESOURCE_CLOCK,
-		SCREEN_RESOURCE_SCREEN,
 		SCREEN_RESOURCE_TRIGGER_OVERVIEW,
 		SCREEN_RESOURCE_DATA_OVERVIEW,
 		SCREEN_RESOURCE_URL,
@@ -538,14 +537,6 @@ class CScreenItem extends CApiService {
 					$mapIds[$screenItem['resourceid']] = $screenItem['resourceid'];
 					break;
 
-				case SCREEN_RESOURCE_SCREEN:
-					if (!$screenItem['resourceid']) {
-						self::exception(ZBX_API_ERROR_PARAMETERS, _('No screen ID provided for screen element.'));
-					}
-
-					$screenIds[$screenItem['resourceid']] = $screenItem['resourceid'];
-					break;
-
 				case SCREEN_RESOURCE_ACTIONS:
 				case SCREEN_RESOURCE_SERVER_INFO:
 				case SCREEN_RESOURCE_SYSTEM_STATUS:
