@@ -139,13 +139,16 @@ function get_events_unacknowledged($db_element, $value_trigger = null, $value_ev
 
 /**
  *
- * @param array  $event								An array of event data.
- * @param string $event['eventid']					Event ID.
- * @param string $event['correlationid']			OK Event correlation ID.
- * @param string $event['userid]					User ID who generated the OK event.
- * @param string $event['name']						Event name.
- * @param string $event['acknowledged']				State of acknowledgement.
- * @param string $backurl							A link back after acknowledgement has been clicked.
+ * @param array  $event                   An array of event data.
+ * @param string $event['eventid']        Event ID.
+ * @param string $event['objectid']       Object ID.
+ * @param string $event['correlationid']  OK Event correlation ID.
+ * @param string $event['userid']         User ID who generated the OK event.
+ * @param string $event['name']           Event name.
+ * @param string $event['acknowledged']   State of acknowledgement.
+ * @param CCOl   $event['opdata']         Operational data with expanded macros.
+ * @param string $event['comments']       Trigger description with expanded macros.
+ * @param string $backurl                 A link back after acknowledgement has been clicked.
  *
  * @return CTableInfo
  */
@@ -163,6 +166,10 @@ function make_event_details($event, $backurl) {
 			_('Event'),
 			$event['name']
 		])
+		->addRow([
+			_('Operational data'),
+			$event['opdata']
+		])
 		->addRow([
 			_('Severity'),
 			getSeverityCell($event['severity'], $config)
@@ -232,7 +239,9 @@ function make_event_details($event, $backurl) {
 
 	$tags = makeTags([$event]);
 
-	$table->addRow([_('Tags'), $tags[$event['eventid']]]);
+	$table
+		->addRow([_('Tags'), $tags[$event['eventid']]])
+		->addRow([_('Description'), (new CDiv(zbx_str2links($event['comments'])))]);
 
 	return $table;
 }
@@ -316,7 +325,7 @@ function make_small_eventlist($startEvent, $backurl) {
 		'preservekeys' => true
 	]);
 	$mediatypes = API::Mediatype()->get([
-		'output' => ['description', 'maxattempts'],
+		'output' => ['name', 'maxattempts'],
 		'mediatypeids' => array_keys($actions['mediatypeids']),
 		'preservekeys' => true
 	]);
@@ -388,7 +397,7 @@ function make_small_eventlist($startEvent, $backurl) {
  *
  * @param array  $trigger                    An array of trigger data.
  * @param string $trigger['triggerid']       Trigger ID to select events.
- * @param string $trigger['description']     Trigger description.
+ * @param string $trigger['comments']        Trigger description.
  * @param string $trigger['url']             Trigger URL.
  * @param string $eventid_till
  * @param string $backurl                    URL to return to.
