@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2017 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,38 +19,23 @@
 **/
 
 
-$hostInventoryWidget = (new CWidget())->setTitle(_('Host inventory'));
-
-/*
- * Overview tab
- */
+// Overview tab.
 $overviewFormList = new CFormList();
 
-$host_name = (new CSpan($data['host']['host']))
-	->addClass(ZBX_STYLE_LINK_ACTION)
-	->setMenuPopup(CMenuPopupHelper::getHost(
-		$data['host'],
-		$data['hostScripts'][$data['host']['hostid']],
-		false
-	));
+$host_name = (new CLinkAction($data['host']['host']))
+	->setMenuPopup(CMenuPopupHelper::getHost($data['host']['hostid'], false));
 
 if ($data['host']['maintenance_status'] == HOST_MAINTENANCE_STATUS_ON) {
-	$maintenance_icon = (new CSpan())
-		->addClass(ZBX_STYLE_ICON_MAINT)
-		->addClass(ZBX_STYLE_CURSOR_POINTER);
-
 	if (array_key_exists($data['host']['maintenanceid'], $data['maintenances'])) {
 		$maintenance = $data['maintenances'][$data['host']['maintenanceid']];
-
-		$hint = $maintenance['name'].' ['.($data['host']['maintenance_type']
-			? _('Maintenance without data collection')
-			: _('Maintenance with data collection')).']';
-
-		if ($maintenance['description']) {
-			$hint .= "\n".$maintenance['description'];
-		}
-
-		$maintenance_icon->setHint($hint);
+		$maintenance_icon = makeMaintenanceIcon($data['host']['maintenance_type'], $maintenance['name'],
+			$maintenance['description']
+		);
+	}
+	else {
+		$maintenance_icon = makeMaintenanceIcon($data['host']['maintenance_type'], _('Inaccessible maintenance'),
+			''
+		);
 	}
 
 	$host_name = (new CSpan([$host_name, $maintenance_icon]))->addClass(ZBX_STYLE_REL_CONTAINER);
@@ -136,13 +121,16 @@ if ($data['host']['description'] !== '') {
 // latest data
 $overviewFormList->addRow(_('Monitoring'),
 	new CHorList([
-		new CLink(_('Web'), 'zabbix.php?action=web.view&hostid='.$data['host']['hostid'].url_param('groupid')),
-		new CLink(_('Latest data'),
-			'latest.php?form=1&select=&show_details=1&filter_set=Filter&hostids[]='.$data['host']['hostid']
+		new CLink(_('Web'),
+			'zabbix.php?action=web.view&hostid='.$data['host']['hostid'].url_param('groupid')
 		),
-		new CLink(_('Triggers'),
-			'tr_status.php?filter_set=1&show_triggers=2&ack_status=1&show_events=1&show_events=0&show_details=1'.
-			'&txt_select=&show_maintenance=1&hostid='.$data['host']['hostid'].url_param('groupid')
+		new CLink(_('Latest data'),
+			(new CUrl('latest.php'))
+				->setArgument('form', '1')
+				->setArgument('select', '')
+				->setArgument('show_details', '1')
+				->setArgument('filter_set', 'Filter')
+				->setArgument('hostids[]', $data['host']['hostid'])
 		),
 		new CLink(_('Problems'),
 			(new CUrl('zabbix.php'))
@@ -151,7 +139,9 @@ $overviewFormList->addRow(_('Monitoring'),
 				->setArgument('filter_set', '1')
 		),
 		new CLink(_('Graphs'), 'charts.php?hostid='.$data['host']['hostid'].url_param('groupid')),
-		new CLink(_('Screens'), 'host_screen.php?hostid='.$data['host']['hostid'].url_param('groupid'))
+		new CLink(_('Screens'),
+			'host_screen.php?hostid='.$data['host']['hostid'].url_param('groupid')
+		)
 	])
 );
 
@@ -161,8 +151,17 @@ if ($data['rwHost']) {
 	$applicationsLink = new CLink(_('Applications'),
 		'applications.php?hostid='.$data['host']['hostid'].url_param('groupid')
 	);
-	$itemsLink = new CLink(_('Items'), 'items.php?filter_set=1&hostid='.$data['host']['hostid'].url_param('groupid'));
-	$triggersLink = new CLink(_('Triggers'), 'triggers.php?hostid='.$data['host']['hostid'].url_param('groupid'));
+
+	$itemsLink = new CLink(_('Items'),
+		(new CUrl('items.php'))
+			->setArgument('filter_set', '1')
+			->setArgument('filter_hostids', [$data['host']['hostid']])
+	);
+	$triggersLink = new CLink(_('Triggers'),
+		(new CUrl('triggers.php'))
+			->setArgument('filter_set', '1')
+			->setArgument('filter_hostids', [$data['host']['hostid']])
+	);
 	$graphsLink = new CLink(_('Graphs'), 'graphs.php?hostid='.$data['host']['hostid'].url_param('groupid'));
 	$discoveryLink = new CLink(_('Discovery'),
 		'host_discovery.php?hostid='.$data['host']['hostid'].url_param('groupid')
@@ -220,8 +219,11 @@ $hostInventoriesTab->addTab('detailsTab', _('Details'), $detailsFormList);
 // append tabs and form
 $hostInventoriesTab->setFooter(makeFormFooter(null, [new CButtonCancel(url_param('groupid'))]));
 
-$hostInventoryWidget->addItem(
-	(new CForm())->addItem($hostInventoriesTab)
-);
-
-return $hostInventoryWidget;
+return (new CWidget())
+	->setTitle(_('Host inventory'))
+	->setWebLayoutMode(CView::getLayoutMode())
+	->setControls((new CList())->addItem(get_icon('fullscreen')))
+	->addItem((new CForm())
+		->setAttribute('aria-labeledby', ZBX_STYLE_PAGE_TITLE)
+		->addItem($hostInventoriesTab)
+	);
