@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2017 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@ if (isset($_REQUEST['screenitemid'])) {
 
 $form = (new CForm('post', $action))
 	->setName('screen_item_form')
+	->setAttribute('aria-labeledby', ZBX_STYLE_PAGE_TITLE)
 	->addVar('screenid', getRequest('screenid'));
 
 if ($data['screen']['templateid'] != 0) {
@@ -84,159 +85,190 @@ if ($this->data['screen']['templateid']) {
 	unset(
 		$screenResources[SCREEN_RESOURCE_DATA_OVERVIEW], $screenResources[SCREEN_RESOURCE_ACTIONS],
 		$screenResources[SCREEN_RESOURCE_EVENTS], $screenResources[SCREEN_RESOURCE_HOST_INFO],
-		$screenResources[SCREEN_RESOURCE_MAP], $screenResources[SCREEN_RESOURCE_SCREEN],
-		$screenResources[SCREEN_RESOURCE_SERVER_INFO], $screenResources[SCREEN_RESOURCE_HOSTGROUP_TRIGGERS],
-		$screenResources[SCREEN_RESOURCE_HOST_TRIGGERS], $screenResources[SCREEN_RESOURCE_SYSTEM_STATUS],
-		$screenResources[SCREEN_RESOURCE_TRIGGER_INFO], $screenResources[SCREEN_RESOURCE_TRIGGER_OVERVIEW]
+		$screenResources[SCREEN_RESOURCE_MAP], $screenResources[SCREEN_RESOURCE_SERVER_INFO],
+		$screenResources[SCREEN_RESOURCE_HOSTGROUP_TRIGGERS], $screenResources[SCREEN_RESOURCE_HOST_TRIGGERS],
+		$screenResources[SCREEN_RESOURCE_SYSTEM_STATUS], $screenResources[SCREEN_RESOURCE_TRIGGER_INFO],
+		$screenResources[SCREEN_RESOURCE_TRIGGER_OVERVIEW]
 	);
 }
 
 $screenFormList = (new CFormList())
-	->addRow(_('Resource'), new CComboBox('resourcetype', $resourceType, 'submit()', $screenResources));
+	->addRow((new CLabel(_('Resource'), 'resourcetype')),
+		(new CComboBox('resourcetype', $resourceType, 'submit()', $screenResources))
+	);
 
 /*
  * Screen item: Graph
  */
 if ($resourceType == SCREEN_RESOURCE_GRAPH) {
-	$caption = '';
-	$id = 0;
+	$graph = false;
 
-	$graphs = API::Graph()->get([
-		'graphids' => $resourceId,
-		'selectHosts' => ['hostid', 'name', 'status'],
-		'output' => API_OUTPUT_EXTEND
-	]);
-	if (!empty($graphs)) {
-		$id = $resourceId;
-		$graph = reset($graphs);
+	if ($resourceId > 0) {
+		$graphs = API::Graph()->get([
+			'output' => ['name'],
+			'selectHosts' => ['name'],
+			'graphids' => $resourceId
+		]);
 
-		order_result($graph['hosts'], 'name');
-		$graph['host'] = reset($graph['hosts']);
+		if ($graphs) {
+			$graph = reset($graphs);
 
-		$caption = $graph['host']['name'].NAME_DELIMITER.$graph['name'];
+			order_result($graph['hosts'], 'name');
+			$graph['host'] = reset($graph['hosts']);
+		}
 	}
 
-	if ($this->data['screen']['templateid']) {
-		$selectButton = (new CButton('select', _('Select')))
-			->addClass(ZBX_STYLE_BTN_GREY)
-			->onClick('javascript: return PopUp("popup.php?srctbl=graphs&srcfld1=graphid&srcfld2=name'.
-				'&dstfrm='.$form->getName().'&dstfld1=resourceid&dstfld2=caption'.
-				'&templated_hosts=1&only_hostid='.$this->data['screen']['templateid'].'");'
-			);
-	}
-	else {
-		$selectButton = (new CButton('select', _('Select')))
-			->addClass(ZBX_STYLE_BTN_GREY)
-			->onClick('javascript: return PopUp("popup.php?srctbl=graphs&srcfld1=graphid&srcfld2=name'.
-				'&dstfrm='.$form->getName().'&dstfld1=resourceid&dstfld2=caption'.
-				'&real_hosts=1&with_graphs=1");');
-	}
-
-	$form->addVar('resourceid', $id);
-	$screenFormList->addRow(_('Graph'), [
-		(new CTextBox('caption', $caption, true))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH),
-		(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
-		$selectButton
-	]);
+	$screenFormList->addRow(
+		(new CLabel(_('Graph'), 'resourceid_ms'))->setAsteriskMark(),
+		(new CMultiSelect([
+			'name' => 'resourceid',
+			'object_name' => 'graphs',
+			'multiple' => false,
+			'data' => $graph
+				? [
+					[
+						'id' => $resourceId,
+						'prefix' => $graph['host']['name'].NAME_DELIMITER,
+						'name' => $graph['name']
+					]
+				]
+				: [],
+			'popup' => [
+				'parameters' => array_merge([
+					'srctbl' => 'graphs',
+					'srcfld1' => 'graphid',
+					'srcfld2' => 'name',
+					'dstfrm' => $form->getName(),
+					'dstfld1' => 'resourceid'
+				], $this->data['screen']['templateid'] ? [
+					'templated_hosts' => '1',
+					'hostid' => $data['screen']['templateid']
+				] : [
+					'real_hosts' => '1',
+					'with_graphs' => '1'
+				]),
+			]
+		]))
+			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+			->setAriaRequired(true)
+	);
 }
 
 /*
  * Screen item: Graph prototype
  */
 elseif ($resourceType == SCREEN_RESOURCE_LLD_GRAPH) {
-	$caption = '';
-	$id = 0;
+	$graph_prototype = false;
 
-	$graphPrototypes = API::GraphPrototype()->get([
-		'output' => ['name'],
-		'graphids' => $resourceId,
-		'selectHosts' => ['name']
-	]);
+	if ($resourceId > 0) {
+		$graph_prototypes = API::GraphPrototype()->get([
+			'output' => ['name'],
+			'selectHosts' => ['name'],
+			'graphids' => $resourceId
+		]);
 
-	if ($graphPrototypes) {
-		$id = $resourceId;
-		$graphPrototype = reset($graphPrototypes);
+		if ($graph_prototypes) {
+			$graph_prototype = reset($graph_prototypes);
 
-		order_result($graphPrototype['hosts'], 'name');
-		$graphPrototype['host'] = reset($graphPrototype['hosts']);
-
-		$caption = $graphPrototype['host']['name'].NAME_DELIMITER.$graphPrototype['name'];
+			order_result($graph_prototype['hosts'], 'name');
+			$graph_prototype['host'] = reset($graph_prototype['hosts']);
+		}
 	}
 
-	if ($this->data['screen']['templateid']) {
-		$selectButton = (new CButton('select', _('Select')))
-			->addClass(ZBX_STYLE_BTN_GREY)
-			->onClick('javascript: return PopUp("popup.php?srctbl=graph_prototypes&srcfld1=graphid&srcfld2=name'.
-				'&dstfrm='.$form->getName().'&dstfld1=resourceid&dstfld2=caption'.
-				'&only_hostid='.$this->data['screen']['templateid'].
-				'&templated_hosts=1");');
-	}
-	else {
-		$selectButton = (new CButton('select', _('Select')))
-			->addClass(ZBX_STYLE_BTN_GREY)
-			->onClick('javascript: return PopUp("popup.php?srctbl=graph_prototypes&srcfld1=graphid&srcfld2=name'.
-				'&dstfrm='.$form->getName().'&dstfld1=resourceid&dstfld2=caption'.
-				'&real_hosts=1");');
-	}
-
-	$form->addVar('resourceid', $id);
-	$screenFormList->addRow(_('Graph prototype'), [
-		(new CTextBox('caption', $caption, true))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH),
-		(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
-		$selectButton
-	]);
-
-	$screenFormList->addRow(_('Max columns'),
-		(new CNumericBox('max_columns', $maxColumns, 3, false, false, false))
-			->setWidth(ZBX_TEXTAREA_NUMERIC_STANDARD_WIDTH)
-	);
+	$screenFormList
+		->addRow(
+			(new CLabel(_('Graph prototype'), 'resourceid_ms'))->setAsteriskMark(),
+			(new CMultiSelect([
+				'name' => 'resourceid',
+				'object_name' => 'graph_prototypes',
+				'multiple' => false,
+				'data' => $graph_prototype
+					? [
+						[
+							'id' => $resourceId,
+							'prefix' => $graph_prototype['host']['name'].NAME_DELIMITER,
+							'name' => $graph_prototype['name']
+						]
+					]
+					: [],
+				'popup' => [
+					'parameters' => array_merge([
+						'srctbl' => 'graph_prototypes',
+						'srcfld1' => 'graphid',
+						'srcfld2' => 'name',
+						'dstfrm' => $form->getName(),
+						'dstfld1' => 'resourceid'
+					], $this->data['screen']['templateid'] ? [
+						'templated_hosts' => '1',
+						'hostid' => $data['screen']['templateid']
+					] : [
+						'real_hosts' => '1'
+					]),
+				]
+			]))
+				->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+				->setAriaRequired()
+		)
+		->addRow(
+			(new CLabel(_('Max columns'), 'max_columns'))->setAsteriskMark(),
+			(new CNumericBox('max_columns', $maxColumns, 3, false, false, false))
+				->setWidth(ZBX_TEXTAREA_NUMERIC_STANDARD_WIDTH)
+				->setAriaRequired()
+		);
 }
 
 /*
  * Screen item: Simple graph
  */
 elseif ($resourceType == SCREEN_RESOURCE_SIMPLE_GRAPH) {
-	$caption = '';
-	$id = 0;
+	$item = false;
 
-	$items = API::Item()->get([
-		'itemids' => $resourceId,
-		'selectHosts' => ['name'],
-		'output' => ['itemid', 'hostid', 'key_', 'name']
-	]);
+	if ($resourceId > 0) {
+		$items = API::Item()->get([
+			'itemids' => $resourceId,
+			'selectHosts' => ['name'],
+			'output' => ['itemid', 'hostid', 'key_', 'name'],
+			'webitems' => true
+		]);
 
-	if ($items) {
-		$items = CMacrosResolverHelper::resolveItemNames($items);
-
-		$id = $resourceId;
-		$item = reset($items);
-		$item['host'] = reset($item['hosts']);
-
-		$caption = $item['host']['name'].NAME_DELIMITER.$item['name_expanded'];
+		if ($items) {
+			$items = CMacrosResolverHelper::resolveItemNames($items);
+			$item = reset($items);
+		}
 	}
 
-	if ($this->data['screen']['templateid']) {
-		$selectButton = (new CButton('select', _('Select')))
-			->addClass(ZBX_STYLE_BTN_GREY)
-			->onClick('javascript: return PopUp("popup.php?srctbl=items&srcfld1=itemid&srcfld2=name'.
-				'&dstfrm='.$form->getName().'&dstfld1=resourceid&dstfld2=caption'.
-				'&templated_hosts=1&only_hostid='.$this->data['screen']['templateid'].
-				'&numeric=1");');
-	}
-	else {
-		$selectButton = (new CButton('select', _('Select')))
-			->addClass(ZBX_STYLE_BTN_GREY)
-			->onClick('javascript: return PopUp("popup.php?srctbl=items&srcfld1=itemid&srcfld2=name'.
-				'&dstfrm='.$form->getName().'&dstfld1=resourceid&dstfld2=caption'.
-				'&real_hosts=1&with_simple_graph_items=1&numeric=1");');
-	}
-
-	$form->addVar('resourceid', $id);
-	$screenFormList->addRow(_('Item'), [
-		(new CTextBox('caption', $caption, true))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH),
-		(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
-		$selectButton
-	]);
+	$screenFormList->addRow(
+		(new CLabel(_('Item'), 'resourceid_ms'))->setAsteriskMark(),
+		(new CMultiSelect([
+			'name' => 'resourceid',
+			'object_name' => 'items',
+			'multiple' => false,
+			'data' => $item
+				? [
+					[
+						'id' => $resourceId,
+						'prefix' => $item['hosts'][0]['name'].NAME_DELIMITER,
+						'name' => $item['name_expanded']
+					]
+				]
+				: [],
+			'popup' => [
+				'parameters' => [
+					'srctbl' => 'items',
+					'srcfld1' => 'itemid',
+					'dstfrm' => $form->getName(),
+					'dstfld1' => 'resourceid',
+					'webitems' => true,
+					'numeric' => true,
+					'hostid' => $data['screen']['templateid'],
+					'real_hosts' => ($data['screen']['templateid'] == 0),
+					'with_simple_graph_items' => ($data['screen']['templateid'] == 0)
+				]
+			]
+		]))
+			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+			->setAriaRequired(true)
+	);
 }
 
 /*
@@ -265,28 +297,52 @@ elseif ($resourceType == SCREEN_RESOURCE_LLD_SIMPLE_GRAPH) {
 	if ($this->data['screen']['templateid']) {
 		$selectButton = (new CButton('select', _('Select')))
 			->addClass(ZBX_STYLE_BTN_GREY)
-			->onClick('javascript: return PopUp("popup.php?srctbl=item_prototypes&srcfld1=itemid&srcfld2=name'.
-				'&dstfrm='.$form->getName().'&dstfld1=resourceid&dstfld2=caption'.
-				'&templated_hosts=1&only_hostid='.$this->data['screen']['templateid'].
-				'&numeric=1");');
+			->onClick('return PopUp("popup.generic",'.
+				CJs::encodeJson([
+					'srctbl' => 'item_prototypes',
+					'srcfld1' => 'itemid',
+					'srcfld2' => 'name',
+					'dstfrm' => $form->getName(),
+					'dstfld1' => 'resourceid',
+					'dstfld2' => 'caption',
+					'templated_hosts' => '1',
+					'only_hostid' => $data['screen']['templateid'],
+					'numeric' => '1'
+				]).', null, this);'
+			);
 	}
 	else {
 		$selectButton = (new CButton('select', _('Select')))
 			->addClass(ZBX_STYLE_BTN_GREY)
-			->onClick('javascript: return PopUp("popup.php?srctbl=item_prototypes&srcfld1=itemid&srcfld2=name'.
-				'&dstfrm='.$form->getName().'&dstfld1=resourceid&dstfld2=caption'.
-				'&real_hosts=1&with_discovery_rule=1&items=1&numeric=1");');
+			->onClick('return PopUp("popup.generic",'.
+				CJs::encodeJson([
+					'srctbl' => 'item_prototypes',
+					'srcfld1' => 'itemid',
+					'srcfld2' => 'name',
+					'dstfrm' => $form->getName(),
+					'dstfld1' => 'resourceid',
+					'dstfld2' => 'caption',
+					'real_hosts' => '1',
+					'with_discovery_rule' => '1',
+					'items' => '1',
+					'numeric' => '1'
+				]).', null, this);'
+			);
 	}
 
 	$form->addVar('resourceid', $id);
-	$screenFormList->addRow(_('Item prototype'), [
-		(new CTextBox('caption', $caption, true))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH),
+	$screenFormList->addRow((new CLabel(_('Item prototype'), 'caption'))->setAsteriskMark(), [
+		(new CTextBox('caption', $caption, true))
+			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+			->setAriaRequired(),
 		(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
 		$selectButton
 	]);
 
-	$screenFormList->addRow(_('Max columns'),
+	$screenFormList->addRow(
+		(new CLabel(_('Max columns'), 'max_columns'))->setAsteriskMark(),
 		(new CNumericBox('max_columns', $maxColumns, 3, false, false, false))
+			->setAriaRequired()
 			->setWidth(ZBX_TEXTAREA_NUMERIC_STANDARD_WIDTH)
 	);
 }
@@ -309,14 +365,23 @@ elseif ($resourceType == SCREEN_RESOURCE_MAP) {
 	}
 
 	$form->addVar('resourceid', $id);
-	$screenFormList->addRow(_('Map'), [
-		(new CTextBox('caption', $caption, true))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH),
+	$screenFormList->addRow((new CLabel(_('Map'), 'caption'))->setAsteriskMark(), [
+		(new CTextBox('caption', $caption, true))
+			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+			->setAriaRequired(),
 		(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
 		(new CButton('select', _('Select')))
 			->addClass(ZBX_STYLE_BTN_GREY)
-			->onClick('javascript: return PopUp("popup.php?srctbl=sysmaps&srcfld1=sysmapid&srcfld2=name'.
-				'&dstfrm='.$form->getName().'&dstfld1=resourceid&dstfld2=caption");'
-		)
+			->onClick('return PopUp("popup.generic",'.
+				CJs::encodeJson([
+					'srctbl' => 'sysmaps',
+					'srcfld1' => 'sysmapid',
+					'srcfld2' => 'name',
+					'dstfrm' => $form->getName(),
+					'dstfld1' => 'resourceid',
+					'dstfld2' => 'caption'
+				]).', null, this);'
+			)
 	]);
 }
 
@@ -324,56 +389,69 @@ elseif ($resourceType == SCREEN_RESOURCE_MAP) {
  * Screen item: Plain text
  */
 elseif ($resourceType == SCREEN_RESOURCE_PLAIN_TEXT) {
-	$caption = '';
-	$id = 0;
+	$item = false;
 
-	$items = API::Item()->get([
-		'itemids' => $resourceId,
-		'selectHosts' => ['name'],
-		'output' => ['itemid', 'hostid', 'key_', 'name']
-	]);
+	if ($resourceId != 0) {
+		$items = API::Item()->get([
+			'output' => ['itemid', 'hostid', 'key_', 'name'],
+			'selectHosts' => ['name'],
+			'itemids' => $resourceId,
+			'webitems' => true
+		]);
 
-	if ($items) {
-		$items = CMacrosResolverHelper::resolveItemNames($items);
-
-		$id = $resourceId;
-		$item = reset($items);
-		$item['host'] = reset($item['hosts']);
-		$caption = $item['host']['name'].NAME_DELIMITER.$item['name_expanded'];
-	}
-
-	$form->addVar('resourceid', $id);
-
-	if ($this->data['screen']['templateid']) {
-		$selectButton = (new CButton('select', _('Select')))
-			->addClass(ZBX_STYLE_BTN_GREY)
-			->onClick('javascript: return PopUp("popup.php?srctbl=items&srcfld1=itemid&srcfld2=name'.
-				'&dstfrm='.$form->getName().'&dstfld1=resourceid&dstfld2=caption'.
-				'&templated_hosts=1&only_hostid='.$this->data['screen']['templateid'].'");'
-			);
-	}
-	else {
-		$selectButton = (new CButton('select', _('Select')))
-			->addClass(ZBX_STYLE_BTN_GREY)
-			->onClick('javascript: return PopUp("popup.php?srctbl=items&srcfld1=itemid&srcfld2=name'.
-				'&dstfrm='.$form->getName().'&dstfld1=resourceid&dstfld2=caption&real_hosts=1");'
-			);
+		if ($items) {
+			$items = CMacrosResolverHelper::resolveItemNames($items);
+			$item = reset($items);
+		}
 	}
 
 	$screenFormList
-		->addRow(_('Item'), [
-			(new CTextBox('caption', $caption, true))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH),
-			(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
-			$selectButton
-		])
-		->addRow(_('Show lines'),
-			(new CNumericBox('elements', $elements, 3))->setWidth(ZBX_TEXTAREA_NUMERIC_STANDARD_WIDTH)
+		->addRow(
+			(new CLabel(_('Item'), 'resourceid_ms'))->setAsteriskMark(),
+			(new CMultiSelect([
+				'name' => 'resourceid',
+				'object_name' => 'items',
+				'multiple' => false,
+				'data' => $item
+					? [
+						[
+							'id' => $resourceId,
+							'prefix' => $item['hosts'][0]['name'].NAME_DELIMITER,
+							'name' => $item['name_expanded']
+						]
+					]
+					: [],
+				'popup' => [
+					'parameters' => [
+						'srctbl' => 'items',
+						'srcfld1' => 'itemid',
+						'dstfrm' => $form->getName(),
+						'dstfld1' => 'resourceid',
+						'hostid' => $data['screen']['templateid'],
+						'real_hosts' => ($data['screen']['templateid'] == 0),
+						'webitems' => true
+					]
+				],
+			]))
+			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+			->setAriaRequired(true)
 		)
-		->addRow(_('Show text as HTML'), (new CCheckBox('style'))->setChecked($style == 1));
+		->addRow((new CLabel(_('Show lines'), 'elements'))->setAsteriskMark(),
+			(new CNumericBox('elements', $elements, 3))
+				->setAriaRequired()
+				->setWidth(ZBX_TEXTAREA_NUMERIC_STANDARD_WIDTH)
+		)
+		->addRow(_('Show text as HTML'),
+			(new CCheckBox('style'))
+				->setChecked($style == 1)
+				->removeId()
+		);
 }
-elseif (in_array($resourceType, [SCREEN_RESOURCE_HOSTGROUP_TRIGGERS, SCREEN_RESOURCE_HOST_TRIGGERS])) {
-	// Screen item: Triggers
 
+/*
+ * Screen item: Triggers
+ */
+elseif (in_array($resourceType, [SCREEN_RESOURCE_HOSTGROUP_TRIGGERS, SCREEN_RESOURCE_HOST_TRIGGERS])) {
 	$data = [];
 
 	if ($resourceType == SCREEN_RESOURCE_HOSTGROUP_TRIGGERS) {
@@ -388,16 +466,28 @@ elseif (in_array($resourceType, [SCREEN_RESOURCE_HOSTGROUP_TRIGGERS, SCREEN_RESO
 			}
 		}
 
-		$screenFormList->addRow(_('Group'),
+		$screenFormList->addRow((new CLabel(_('Group'), 'resourceid_ms')),
 			(new CMultiSelect([
 				'name' => 'resourceid',
-				'objectName' => 'hostGroup',
-				'data' => $data ? [['id' => $data['groupid'], 'name' => $data['name']]] : null,
-				'defaultValue' => 0,
-				'selectedLimit' => 1,
+				'object_name' => 'hostGroup',
+				'multiple' => false,
+				'default_value' => 0,
+				'data' => $data
+					? [
+						[
+							'id' => $data['groupid'],
+							'name' => $data['name']
+						]
+					]
+					: [],
 				'popup' => [
-					'parameters' => 'srctbl=host_groups&dstfrm='.$form->getName().'&dstfld1=resourceid'.
-						'&srcfld1=groupid'
+					'parameters' => [
+						'srctbl' => 'host_groups',
+						'srcfld1' => 'groupid',
+						'dstfrm' => $form->getName(),
+						'dstfld1' => 'resourceid',
+						'real_hosts' => true
+					]
 				]
 			]))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
 		);
@@ -414,23 +504,37 @@ elseif (in_array($resourceType, [SCREEN_RESOURCE_HOSTGROUP_TRIGGERS, SCREEN_RESO
 			}
 		}
 
-		$screenFormList->addRow(_('Host'),
+		$screenFormList->addRow((new CLabel(_('Host'), 'resourceid_ms')),
 			(new CMultiSelect([
 				'name' => 'resourceid',
-				'objectName' => 'hosts',
-				'data' => $data ? [['id' => $data['hostid'], 'name' => $data['name']]] : null,
-				'defaultValue' => 0,
-				'selectedLimit' => 1,
+				'object_name' => 'hosts',
+				'multiple' => false,
+				'default_value' => 0,
+				'data' => $data
+					? [
+						[
+							'id' => $data['hostid'],
+							'name' => $data['name']
+						]
+					]
+					: [],
 				'popup' => [
-					'parameters' => 'srctbl=hosts&dstfrm='.$form->getName().'&dstfld1=resourceid'.
-						'&srcfld1=hostid'
+					'parameters' => [
+						'srctbl' => 'hosts',
+						'srcfld1' => 'hostid',
+						'dstfrm' => $form->getName(),
+						'dstfld1' => 'resourceid'
+					]
 				]
 			]))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
 		);
 	}
 
-	$screenFormList->addRow(_('Show lines'),
-		(new CNumericBox('elements', $elements, 3))->setWidth(ZBX_TEXTAREA_NUMERIC_STANDARD_WIDTH)
+	$screenFormList->addRow(
+		(new CLabel(_('Show lines'), 'elements'))->setAsteriskMark(),
+		(new CNumericBox('elements', $elements, 3))
+			->setAriaRequired()
+			->setWidth(ZBX_TEXTAREA_NUMERIC_STANDARD_WIDTH)
 	);
 	$screenFormList->addRow(
 		_('Sort triggers by'),
@@ -446,8 +550,11 @@ elseif (in_array($resourceType, [SCREEN_RESOURCE_HOSTGROUP_TRIGGERS, SCREEN_RESO
  * Screen item: Action log
  */
 elseif ($resourceType == SCREEN_RESOURCE_ACTIONS) {
-	$screenFormList->addRow(_('Show lines'),
-		(new CNumericBox('elements', $elements, 3))->setWidth(ZBX_TEXTAREA_NUMERIC_STANDARD_WIDTH)
+	$screenFormList->addRow(
+		(new CLabel(_('Show lines'), 'elements'))->setAsteriskMark(),
+		(new CNumericBox('elements', $elements, 3))
+			->setAriaRequired()
+			->setWidth(ZBX_TEXTAREA_NUMERIC_STANDARD_WIDTH)
 	);
 	$screenFormList->addRow(
 		_('Sort entries by'),
@@ -468,8 +575,11 @@ elseif ($resourceType == SCREEN_RESOURCE_ACTIONS) {
  * Screen item: History of events
  */
 elseif ($resourceType == SCREEN_RESOURCE_EVENTS) {
-	$screenFormList->addRow(_('Show lines'),
-		(new CNumericBox('elements', $elements, 3))->setWidth(ZBX_TEXTAREA_NUMERIC_STANDARD_WIDTH)
+	$screenFormList->addRow(
+		(new CLabel(_('Show lines'), 'elements'))->setAsteriskMark(),
+		(new CNumericBox('elements', $elements, 3))
+			->setAriaRequired()
+			->setWidth(ZBX_TEXTAREA_NUMERIC_STANDARD_WIDTH)
 	);
 	$form->addVar('resourceid', 0);
 }
@@ -491,64 +601,55 @@ elseif (in_array($resourceType, [SCREEN_RESOURCE_TRIGGER_OVERVIEW, SCREEN_RESOUR
 		}
 	}
 
-	$screenFormList->addRow(_('Group'),
+	$screenFormList->addRow((new CLabel(_('Group'), 'resourceid_ms'))->setAsteriskMark(),
 		(new CMultiSelect([
 			'name' => 'resourceid',
-			'objectName' => 'hostGroup',
-			'data' => $data ? [['id' => $data['groupid'], 'name' => $data['name']]] : null,
-			'selectedLimit' => 1,
+			'object_name' => 'hostGroup',
+			'multiple' => false,
+			'data' => $data
+				? [
+					[
+						'id' => $data['groupid'],
+						'name' => $data['name']
+					]
+				]
+				: [],
 			'popup' => [
-				'parameters' => 'srctbl=host_groups&dstfrm='.$form->getName().'&dstfld1=resourceid'.
-					'&srcfld1=groupid'
+				'parameters' => [
+					'srctbl' => 'host_groups',
+					'srcfld1' => 'groupid',
+					'dstfrm' => $form->getName(),
+					'dstfld1' => 'resourceid',
+					'real_hosts' => true
+				]
 			]
-		]))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
-	);
-	$screenFormList->addRow(_('Application'),
-		(new CTextBox('application', $application, false, 255))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
-	);
+		]))
+			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+			->setAriaRequired()
+	)
+	->addRow(_('Application'), [
+		(new CTextBox('application', $application))
+			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH),
+		(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
+		(new CButton('application_select', _('Select')))
+			->addClass(ZBX_STYLE_BTN_GREY)
+			->onClick('return PopUp("popup.generic", '.
+				CJs::encodeJson([
+					'srctbl' => 'applications',
+					'srcfld1' => 'name',
+					'dstfrm' => $form->getName(),
+					'dstfld1' => 'application',
+					'real_hosts' => '1',
+					'with_applications' => '1'
+				]).', null, this);'
+			)
+	]);
 }
 
 /*
- * Screen item: Screens
+ * Screen item: Host info
  */
-elseif ($resourceType == SCREEN_RESOURCE_SCREEN) {
-	$caption = '';
-	$id = 0;
-
-	if ($resourceId > 0) {
-		$db_screens = DBselect('SELECT s.screenid,s.name FROM screens s WHERE s.screenid='.zbx_dbstr($resourceId));
-
-		while ($row = DBfetch($db_screens)) {
-			$screen = API::Screen()->get([
-				'screenids' => $row['screenid'],
-				'output' => ['screenid']
-			]);
-			if (empty($screen)) {
-				continue;
-			}
-			if (check_screen_recursion($_REQUEST['screenid'], $row['screenid'])) {
-				continue;
-			}
-
-			$caption = $row['name'];
-			$id = $resourceId;
-		}
-	}
-
-	$form->addVar('resourceid', $id);
-	$screenFormList->addRow(_('Screen'), [
-		(new CTextBox('caption', $caption, true))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH),
-		(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
-		(new CButton('select', _('Select')))
-			->addClass(ZBX_STYLE_BTN_GREY)
-			->onClick('javascript: return PopUp("popup.php?srctbl=screens2&srcfld1=screenid&srcfld2=name'.
-				'&dstfrm='.$form->getName().'&dstfld1=resourceid&dstfld2=caption'.
-				'&screenid='.$_REQUEST['screenid'].'");')
-	]);
-}
 elseif ($resourceType == SCREEN_RESOURCE_HOST_INFO || $resourceType == SCREEN_RESOURCE_TRIGGER_INFO) {
-	// Screen item: Host info
-
 	$data = [];
 
 	if ($resourceId > 0) {
@@ -562,16 +663,28 @@ elseif ($resourceType == SCREEN_RESOURCE_HOST_INFO || $resourceType == SCREEN_RE
 		}
 	}
 
-	$screenFormList->addRow(_('Group'),
+	$screenFormList->addRow((new CLabel(_('Group'), 'resourceid_ms')),
 		(new CMultiSelect([
 			'name' => 'resourceid',
-			'objectName' => 'hostGroup',
-			'data' => $data ? [['id' => $data['groupid'], 'name' => $data['name']]] : null,
-			'defaultValue' => 0,
-			'selectedLimit' => 1,
+			'object_name' => 'hostGroup',
+			'multiple' => false,
+			'default_value' => 0,
+			'data' => $data
+				? [
+					[
+						'id' => $data['groupid'],
+						'name' => $data['name']
+					]
+				]
+				: [],
 			'popup' => [
-				'parameters' => 'srctbl=host_groups&dstfrm='.$form->getName().'&dstfld1=resourceid'.
-					'&srcfld1=groupid'
+				'parameters' => [
+					'srctbl' => 'host_groups',
+					'srcfld1' => 'groupid',
+					'dstfrm' => $form->getName(),
+					'dstfld1' => 'resourceid',
+					'real_hosts' => true
+				]
 			]
 		]))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
 	);
@@ -581,24 +694,7 @@ elseif ($resourceType == SCREEN_RESOURCE_HOST_INFO || $resourceType == SCREEN_RE
  * Screen item: Clock
  */
 elseif ($resourceType == SCREEN_RESOURCE_CLOCK) {
-	$caption = getRequest('caption', '');
-
-	if (zbx_empty($caption) && TIME_TYPE_HOST == $style && $resourceId > 0) {
-		$items = API::Item()->get([
-			'output' => ['itemid', 'hostid', 'key_', 'name'],
-			'selectHosts' => ['name'],
-			'itemids' => $resourceId,
-			'webitems' => true
-		]);
-
-		if ($items) {
-			$items = CMacrosResolverHelper::resolveItemNames($items);
-
-			$item = reset($items);
-			$host = reset($item['hosts']);
-			$caption = $host['name'].NAME_DELIMITER.$item['name_expanded'];
-		}
-	}
+	$item = false;
 
 	$screenFormList->addRow(_('Time type'), new CComboBox('style', $style, 'submit()', [
 		TIME_TYPE_LOCAL => _('Local time'),
@@ -606,30 +702,51 @@ elseif ($resourceType == SCREEN_RESOURCE_CLOCK) {
 		TIME_TYPE_HOST => _('Host time')
 	]));
 
-	if (TIME_TYPE_HOST == $style) {
-		$form->addVar('resourceid', $resourceId);
+	if ($style == TIME_TYPE_HOST) {
+		if ($resourceId > 0) {
+			$items = API::Item()->get([
+				'output' => ['itemid', 'hostid', 'key_', 'name'],
+				'selectHosts' => ['name'],
+				'itemids' => $resourceId,
+				'webitems' => true
+			]);
 
-		if ($this->data['screen']['templateid']) {
-			$selectButton = (new CButton('select', _('Select')))
-				->addClass(ZBX_STYLE_BTN_GREY)
-				->onClick("javascript: return PopUp('popup.php?dstfrm=".$form->getName().
-					'&dstfld1=resourceid&dstfld2=caption&srctbl=items&srcfld1=itemid&srcfld2=name&templated_hosts=1'.
-					'&only_hostid='.$this->data['screen']['templateid']."');");
+			if ($items) {
+				$items = CMacrosResolverHelper::resolveItemNames($items);
+				$item = reset($items);
+			}
 		}
-		else {
-			$selectButton = (new CButton('select', _('Select')))
-				->addClass(ZBX_STYLE_BTN_GREY)
-				->onClick("javascript: return PopUp('popup.php?dstfrm=".$form->getName().'&dstfld1=resourceid'.
-					"&dstfld2=caption&srctbl=items&srcfld1=itemid&srcfld2=name&real_hosts=1');");
-		}
-		$screenFormList->addRow(_('Item'), [
-			(new CTextBox('caption', $caption, true))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH),
-			(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
-			$selectButton
-		]);
-	}
-	else {
-		$form->addVar('caption', $caption);
+
+		$screenFormList->addRow(
+			(new CLabel(_('Item'), 'resourceid_ms'))->setAsteriskMark(),
+			(new CMultiSelect([
+				'name' => 'resourceid',
+				'object_name' => 'items',
+				'multiple' => false,
+				'data' => $item
+					? [
+						[
+							'id' => $resourceId,
+							'prefix' => $item['hosts'][0]['name'].NAME_DELIMITER,
+							'name' => $item['name_expanded']
+						]
+					]
+					: [],
+				'popup' => [
+					'parameters' => [
+						'srctbl' => 'items',
+						'srcfld1' => 'itemid',
+						'dstfrm' => $form->getName(),
+						'dstfld1' => 'resourceid',
+						'hostid' => $data['screen']['templateid'],
+						'real_hosts' => ($data['screen']['templateid'] == 0),
+						'webitems' => true
+					]
+				]
+			]))
+				->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+				->setAriaRequired(true)
+		);
 	}
 }
 
@@ -657,7 +774,9 @@ elseif ($resourceType != SCREEN_RESOURCE_CLOCK) {
 }
 
 if (in_array($resourceType, [SCREEN_RESOURCE_URL])) {
-	$screenFormList->addRow(_('URL'), (new CTextBox('url', $url))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH));
+	$screenFormList->addRow((new CLabel(_('URL'), 'url'))->setAsteriskMark(),
+		(new CTextBox('url', $url))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)->setAriaRequired()
+	);
 }
 else {
 	$form->addVar('url', '');
@@ -698,7 +817,7 @@ if (in_array($resourceType, $resourcesWithHAlign)) {
 	$screenFormList->addRow(_('Horizontal align'),
 		(new CRadioButtonList('halign', (int) $halign))
 			->addValue(_('Left'), HALIGN_LEFT)
-			->addValue(_('Center'), HALIGN_CENTER)
+			->addValue(_('Centre'), HALIGN_CENTER)
 			->addValue(_('Right'), HALIGN_RIGHT)
 			->setModern(true)
 	);
@@ -714,11 +833,17 @@ $screenFormList->addRow(_('Vertical align'),
 		->addValue(_('Bottom'), VALIGN_BOTTOM)
 		->setModern(true)
 );
-$screenFormList->addRow(_('Column span'),
-	(new CNumericBox('colspan', $colspan, 3))->setWidth(ZBX_TEXTAREA_NUMERIC_STANDARD_WIDTH)
+$screenFormList->addRow(
+	(new CLabel(_('Column span'), 'colspan'))->setAsteriskMark(),
+	(new CNumericBox('colspan', $colspan, 3))
+		->setAriaRequired()
+		->setWidth(ZBX_TEXTAREA_NUMERIC_STANDARD_WIDTH)
 );
-$screenFormList->addRow(_('Row span'),
-	(new CNumericBox('rowspan', $rowspan, 3))->setWidth(ZBX_TEXTAREA_NUMERIC_STANDARD_WIDTH)
+$screenFormList->addRow(
+	(new CLabel(_('Row span'), 'rowspan'))->setAsteriskMark(),
+	(new CNumericBox('rowspan', $rowspan, 3))
+		->setAriaRequired()
+		->setWidth(ZBX_TEXTAREA_NUMERIC_STANDARD_WIDTH)
 );
 
 // dynamic addon
