@@ -471,6 +471,23 @@ function stripslashes(str) {
 	});
 }
 
+/**
+ * Function to remove preloader and moves focus to IU element that was clicked to open it.
+ *
+ * @param string   id			Preloader identifier.
+ * @param {object} xhr			(optional) XHR request that must be aborted.
+ */
+function overlayPreloaderDestroy(id, xhr) {
+	if (typeof id !== 'undefined') {
+		if (typeof xhr !== 'undefined') {
+			xhr.abort();
+		}
+
+		jQuery('#' + id).remove();
+		removeFromOverlaysStack(id);
+	}
+}
+
 /**
  * Function to close overlay dialogue and moves focus to IU element that was clicked to open it.
  *
@@ -485,13 +502,13 @@ function overlayDialogueDestroy(dialogueid, xhr) {
 
 		jQuery('[data-dialogueid='+dialogueid+']').remove();
 
-		if (!jQuery('[data-dialogueid]').length) {
-			jQuery('body').css({'overflow': ''});
-			jQuery('body[style=""]').removeAttr('style');
-		}
-
 		removeFromOverlaysStack(dialogueid);
 		jQuery.publish('overlay.close', {dialogueid: dialogueid});
+
+		if (!jQuery('[data-dialogueid]').length) {
+			jQuery('body').css('overflow', jQuery('body').data('overflow'));
+			jQuery('body').removeData('overflow');
+		}
 	}
 }
 
@@ -536,6 +553,11 @@ function getOverlayDialogueId() {
  * @return {bool}
  */
 function overlayDialogue(params, trigger_elmnt, xhr) {
+	if (!jQuery('[data-dialogueid]').length) {
+		jQuery('body').data('overflow', jQuery('body').css('overflow'));
+		jQuery('body').css('overflow', 'hidden');
+	}
+
 	var button_focused = null,
 		cancel_action = null,
 		submit_btn = null,
@@ -586,6 +608,18 @@ function overlayDialogue(params, trigger_elmnt, xhr) {
 	}
 
 	var center_overlay_dialog = function() {
+			var body = jQuery('.overlay-dialogue-body', overlay_dialogue),
+				body_scroll_height = body[0].scrollHeight,
+				body_height = body.height();
+
+			if (body_height != Math.floor(body_height)) {
+				// The body height is often about a half pixel less than the height.
+				body_height = Math.floor(body_height) + 1;
+			}
+
+			// A fix for IE and Edge to stop popup width flickering when having vertical scrollbar.
+			body.css('overflow-y', body_scroll_height > body_height ? 'scroll' : 'hidden');
+
 			overlay_dialogue.css({
 				'left': Math.round((jQuery(window).width() - jQuery(overlay_dialogue).outerWidth()) / 2) + 'px',
 				'top': overlay_dialogue.hasClass('sticked-to-top')
@@ -631,7 +665,7 @@ function overlayDialogue(params, trigger_elmnt, xhr) {
 		}
 
 		if ('enabled' in obj && obj.enabled === false) {
-			button.attr('disabled', 'disabled');
+			button.prop('disabled', true);
 		}
 
 		if ('focused' in obj && obj.focused === true) {
@@ -714,8 +748,6 @@ function overlayDialogue(params, trigger_elmnt, xhr) {
 		}
 	});
 
-	jQuery('body').css({'overflow': 'hidden'});
-
 	if (button_focused !== null) {
 		button_focused.focus();
 	}
@@ -857,7 +889,15 @@ function executeScript(hostid, scriptid, confirmation, trigger_elmnt) {
 						l = n['name'].indexOf('[', r + 1);
 						r = n['name'].indexOf(']', r + 1);
 
-						if (l == -1 || r == -1 || r <= l) {
+						if (l + 1 == r) {
+							if (typeof curr_json[key] === 'undefined') {
+								curr_json[key] = [];
+							}
+
+							curr_json[key].push(n['value']);
+							break;
+						}
+						else if (l == -1 || r == -1 || r <= l) {
 							curr_json[key] = n['value']
 							break;
 						}
@@ -932,3 +972,108 @@ function parseUrlString(url) {
 		'pairs': pairs
 	};
 }
+
+/**
+ * Message formatting function.
+ *
+ * @param {string}       type            Message type. ('good'|'bad'|'warning')
+ * @param {string|array} messages        Array with details messages or message string with normal font.
+ * @param {string}       title           Larger font title.
+ * @param {bool}         show_close_box  Show close button.
+ * @param {bool}         show_details    Show details on opening.
+ *
+ * @return {string}
+ */
+function makeMessageBox(type, messages, title, show_close_box, show_details) {
+	var classes = {good: 'msg-good', bad: 'msg-bad', warning: 'msg-warning'},
+		msg_class = classes[type];
+
+	if (typeof msg_class === 'undefined') {
+		return jQuery('<output>').text(Array.isArray(messages) ? messages.join(' ') : messages);
+	}
+
+	if (typeof title === 'undefined') {
+		title = null;
+	}
+	if (typeof show_close_box === 'undefined') {
+		show_close_box = true;
+	}
+	if (typeof show_details === 'undefined') {
+		show_details = false;
+	}
+
+	var	$list = jQuery('<ul>'),
+		$msg_details = jQuery('<div>')
+			.addClass('msg-details')
+			.append($list),
+		aria_labels = {good: t('Success message'), bad: t('Error message'), warning: t('Warning message')},
+		$msg_box = jQuery('<output>')
+			.addClass(msg_class).attr('role', 'contentinfo')
+			.attr('aria-label', aria_labels[type]),
+		$details_arrow = jQuery('<span>')
+			.attr('id', 'details-arrow')
+			.addClass(show_details ? 'arrow-up' : 'arrow-down'),
+		$link_details = jQuery('<a>')
+			.text(t('Details') + ' ')
+			.addClass('link-action')
+			.attr('href', 'javascript:void(0)')
+			.attr('role', 'button')
+			.append($details_arrow)
+			.attr('aria-expanded', show_details ? 'true' : 'false');
+
+		$link_details.click(function() {
+			showHide(jQuery(this)
+				.siblings('.msg-details')
+				.find('.msg-details-border')
+			);
+			jQuery('#details-arrow', jQuery(this)).toggleClass('arrow-up arrow-down');
+			jQuery(this).attr('aria-expanded', jQuery(this)
+				.find('.arrow-down')
+				.length == 0
+			);
+		});
+
+	if (title !== null) {
+		$msg_box.prepend($link_details);
+		jQuery('<span>')
+			.text(title)
+			.appendTo($msg_box);
+
+		$list.addClass('msg-details-border');
+
+		if (!show_details) {
+			$list.hide();
+		}
+	}
+
+	if (Array.isArray(messages) && messages.length > 0) {
+		jQuery.map(messages, function(message) {
+			jQuery('<li>')
+				.text(message)
+				.appendTo($list);
+			return null;
+		});
+
+		$msg_box.append($msg_details);
+	}
+	else {
+		jQuery('<li>')
+			.text(messages ? messages : ' ')
+			.appendTo($list);
+		$msg_box.append($msg_details);
+	}
+
+	if (show_close_box) {
+		var $button = jQuery('<button>')
+				.addClass('overlay-close-btn')
+				.attr('title', t('Close'))
+				.click(function() {
+					jQuery(this)
+						.closest('.' + classes[index])
+						.remove();
+				});
+		$msg_box.append($button);
+	}
+
+	return $msg_box;
+}
